/*
	HASMenuWindows.h from Hsoi's App Shell.   1995-1997 John C. Daub.  All rights reserved.
	
	This file is the header file to HASMenuWindows.c.  it contains the various constants,
	resource IDs, #defines/macros, and function prototypes for using the functions in
	HASMenuWindows.c
	
	As with HASMenuWindows.c, HASMenuWindows.h is based upon the work of Scott Knaster
	and Keith Rollin from their book "Macintosh Programming Secrets" 2nd ed. Chapter 7.
	Things have been modified to work with Hsoi's App Shell, such as writing 
	UniversalProcPtr's for callback routines, etc.
*/

#ifndef __MIXEDMODE__
#include <MixedMode.h>
#endif
#ifndef __WINDOWS__
#include <Windows.h>
#endif
#ifndef __DEVICES__
#include <Devices.h>
#endif


#ifdef __cplusplus
extern "C" {
#endif

#if PRAGMA_ALIGN_SUPPORTED
#pragma options align=mac68k
#endif

//========================
// Constants
//========================

enum {
	kMinHeight			=	100,
	kMinWidth			=	100,
	kGapBetweenWindows	=	2
};

enum {
	kVerticalStagger	=	20,
	kHorizontalStagger	=	10,
	kWindowsPerDiagonal	=	8
};


//========================
// Resource IDs
//========================


//========================
// #defines
//========================


//========================
// UPP Info
//========================

// callback procs

typedef void (*HsoiSetUpProcPtr)(void);
typedef	void (*HsoiWindowActionProcPtr)(WindowRef theWindow);
typedef void (*HsoiFinishUpProcPtr)(void);

// upp proc info

enum {
	uppHsoiSetUpProcInfo = kCStackBased
};

enum {
	uppHsoiWindowActionProcInfo = kCStackBased
		| STACK_ROUTINE_PARAMETER(1,SIZE_CODE(sizeof(WindowRef /*theWindow*/)))
};

enum {
	uppHsoiFinishUpProcInfo = kCStackBased
};

#if GENERATINGCFM

typedef UniversalProcPtr HsoiSetUpUPP;
typedef UniversalProcPtr HsoiWindowActionUPP;
typedef UniversalProcPtr HsoiFinishUpUPP;

#define NewHsoiSetUpProc( userRoutine ) \
	(HsoiSetUpUPP) NewRoutineDescriptor( (ProcPtr) (userRoutine), uppHsoiSetUpProcInfo, GetCurrentArchitecture() )
#define NewHsoiWindowActionProc( userRoutine ) \
	(HsoiWindowActionUPP) NewRoutineDescriptor( (ProcPtr) (userRoutine), uppHsoiWindowActionProcInfo, GetCurrentArchitecture() )
#define NewHsoiFinishUpProc( userRoutine ) \
	(HsoiFinishUpUPP) NewRoutineDescriptor( (ProcPtr) (userRoutine), uppHsoiFinishUpProcInfo, GetCurrentArchitecture() )

#define CallHsoiSetUpProc( userRoutine ) \
	CallUniversalProc( (UniversalProcPtr)(userRoutine), uppHsoiSetUpProcInfo )
#define CallHsoiWindowActionProc( userRoutine, whichWindow ) \
	CallUniversalProc( (UniversalProcPtr)(userRoutine), uppHsoiWindowActionProcInfo, (whichWindow) )
#define CallHsoiFinishUpProc( userRoutine ) \
	CallUniversalProc( (UniversalProcPtr)(userRoutine), uppHsoiFinishUpProcInfo )

#else // !GENERATINGCFM, i.e. basically, straight-up 68k stuff

typedef HsoiSetUpProcPtr			HsoiSetUpUPP;
typedef HsoiWindowActionProcPtr		HsoiWindowActionUPP;
typedef HsoiFinishUpProcPtr			HsoiFinishUpUPP;

#define NewHsoiSetUpProc( userRoutine ) \
	((HsoiSetUpUPP)(userRoutine))
#define NewHsoiWindowActionProc( userRoutine ) \
	((HsoiWindowActionUPP)(userRoutine))
#define NewHsoiFinishUpProc( userRoutine ) \
	((HsoiFinishUpUPP)(userRoutine))

#define CallHsoiSetUpProc( userRoutine ) \
	(*(userRoutine))()
#define CallHsoiWindowActionProc( userRoutine, whichWindow ) \
	(*(userRoutine))((whichWindow))
#define CallHsoiFinishUpProc( userRoutine ) \
	(*(userRoutine))()

#endif

//========================
// Function Prototypes
//========================


void		HsoiInitMenuWindowUPPs( void );
void		HsoiReadjustAfterStackTile( WindowRef window );

long		HsoiGetNextWindow( WindowRef window );
void		HsoiSetNextWindow( WindowRef window, long data );

void		HsoiDoTileWindows(void);
void		HsoiTileSetup(void);
void		HsoiTileTheWindow(WindowRef theWindow);

void		HsoiDoStackWindows(void);
void		HsoiStackSetup(void);
void		HsoiStackTheWindow(WindowRef theWindow);

void		HsoiDoSelectFromWindowsMenu(short menuItem);
void		HsoiLookForSelectedWindow(WindowRef theWindow);

void		HsoiAddWindowToMenu(WindowRef theWindow);
void		HsoiRemoveWindowFromMenu(WindowRef theWindow);

void		HsoiForEachWindowDo(HsoiSetUpUPP theStarter,
							HsoiWindowActionUPP theDoer,
							HsoiFinishUpUPP theEnder);
void		HsoiForEachWindowPerScreenDo(HsoiSetUpUPP theStarter,
										HsoiWindowActionUPP theDoer,
										HsoiFinishUpUPP theEnder);
void		HsoiCountWindows(WindowRef theWindow);

WindowRef	HsoiGetPreviouslyCreatedWindow(WindowRef theWindow);
void		HsoiLookForPreviousWindow(WindowRef theWindow);

short		HsoiGetMenuItemForWindow(WindowRef theWindow);
void		HsoiCountSomeWindow(WindowRef theWindow);

void		HsoiSetWindowBounds(WindowRef theWindow, Rect newBounds);

Rect		HsoiGetWindowContentRect(WindowRef window);
Rect		HsoiGetWindowStructureRect(WindowRef window);
Rect		HsoiGetWindowDeviceRectNMB(WindowRef window);
Rect		HsoiGetWindowDeviceRect(WindowRef window);
GDHandle	HsoiGetWindowDevice(WindowRef window);
GDHandle	HsoiGetRectDevice(Rect globalRect);
void		HsoiLocalToGlobalRect(Rect *aRect);
Point		HsoiGetGlobalTopLeft(WindowRef window);
Rect		HsoiGetMainScreenRect(void);
void		HsoiAdjustWindowsMenu( WindowRef window );


#if PRAGMA_ALIGN_SUPPORTED
#pragma options align=reset
#endif

#ifdef __cplusplus
}
#endif